;; -*-byte-compile-dynamic: t;-*-
;;; skk-eeie.el --- SKK Easy English Imputting Environment.
;; Copyright (C) 1999 Mikio Nakajima <minakaji@osaka.email.ne.jp>

;; Author: Mikio Nakajima <minakaji@osaka.email.ne.jp>
;; Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
;; Version: $Id:$
;; Keywords: japanese
;; Last Modified: $Date:$

;; This file is not part of SKK yet.

;; SKK is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either versions 2, or (at your option)
;; any later version.

;; SKK is distributed in the hope that it will be useful
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with SKK, see the file COPYING.  If not, write to the Free
;; Software Foundation Inc., 59 Temple Place - Suite 330, Boston,
;; MA 02111-1307, USA.

;;; Commentary:
;;; Not implemented yet.

;;; Code:
(eval-and-compile
  ;; APEL 9.20 or later required.
  (require 'alist)
  ;; Elib version 1.0 required.
  (require 'stack-m)
  (require 'skk) (require 'skk-look) )

(defcustom skk-eeie-completion-forward-key "\C-\t"
  "*"
  :type 'string
  :group 'skk-eeie )

(defcustom skk-eeie-completion-backward-key "\C-\S-\t"
  "*"
  :type 'string
  :group 'skk-eeie )

(defcustom skk-eeie-mode-string " Eeie"
  "*"
  :type 'string
  :group 'skk )

(defcustom skk-eeie-cursor-color "PowderBlue"
  "*eeie $B%b!<%I$r<($9%+!<%=%k?'!#(B
skk-use-color-cursor $B$,(B non-nil $B$N$H$-$K;HMQ$5$l$k!#(B" 
  :type 'string
  :group 'skk )

(skk-deflocalvar skk-eeie-completion-stack nil)
(skk-deflocalvar skk-eeie-completing-word nil)
(skk-deflocalvar skk-eeie-completed-list nil) 
(skk-deflocalvar skk-eeie-start nil)
(defvar skk-eeie-mode nil)

(defvar skk-eeie-mode-map nil "*SKK eeie $B%b!<%I$N%-!<%^%C%W!#(B")
(or skk-eeie-mode-map 
    (let ((map (make-sparse-keymap)))
      (define-key map skk-eeie-completion-forward-key 'skk-eeie-forward)
      (define-key map skk-eeie-completion-backward-key 'skk-eeie-completion-backward)
      (skk-define-menu-bar-map map)
      (setq skk-eeie-mode-map map) ))

(setq-default minor-mode-map-alist
	      (put-alist 'skk-eeie-mode
			 ;; each element of minor-mode-alist is not cons cell.
			 skk-eeie-mode-map minor-mode-map-alist) )

;;;###autoload
(defun skk-eeie-mode (arg)
  ""
  (interactive "*P")
  (skk-kakutei)
  (prog1
      (setq skk-eeie-mode
	    (if (null arg) (not skk-eeie-mode)
	      (> (prefix-numeric-value arg) 0) ))
    (if (not skk-eeie-mode)
	(setq skk-mode nil
	      skk-abbrev-mode nil
	      skk-latin-mode nil
	      skk-j-mode nil
	      skk-jisx0208-latin-mode nil
	      skk-katakana nil
	      skk-input-mode-string skk-ascii-mode-string )
      (setq skk-mode t
	    skk-abbrev-mode nil
	    skk-latin-mode nil
	    skk-j-mode nil
	    skk-jisx0208-latin-mode nil
	    ;; j's sub mode.
	    skk-katakana nil
	    skk-input-mode-string skk-eeie-mode-string )
      (skk-set-cursor-color skk-eeie-cursor-color) )
    (force-mode-line-update) ))
 
(defun skk-eeie-completion-forward()
  (interactive)
  (let ((first
	 (null (memq last-command '(skk-eeie-completion-forward
				    skk-eeie-completion-backward ))))
	c-word )
    (if (not first)
	nil
      (setq skk-eeie-completion-stack (stack-create))
      (or (markerp 'skk-eeie-start)
	  (setq skk-eeie-start (make-marker)) )
      (set-marker skk-eeie-start 
		  (skk-save-point (progn (backward-word 1) (point))) )
      (setq skk-eeie-completing-word (buffer-substring-no-properties
				      (point) skk-eeie-start ))
      (and (string= skk-eeie-completing-word "")
	   (skk-error "$B6uJ8;z$+$iJd40$9$k$3$H$O$G$-$^$;$s!*(B"
		      "Cannot complete an empty string!" ))
      (setq skk-eeie-completed-list (skk-look-1 skk-eeie-completing-word)) )
    (setq c-word (car skk-eeie-completed-list))
    (setq skk-eeie-completed-list (cdr skk-eeie-completed-list))
    (if (not c-word)
	(if skk-japanese-message-and-error
	    (error "\"%s\" $B$GJd40$9$Y$-8+=P$78l$O(B%s$B$"$j$^$;$s(B"
		   skk-eeie-completing-word (if first "" "$BB>$K(B") )
	  (error "No %scompletions for \"%s\""
		 (if first "" "more ") skk-eeie-completing-word ))
      (stack-push skk-eeie-completion-stack c-word)
      (delete-region skk-eeie-start (point))
      (insert c-word) )))

(defun skk-eeie-completion-backward()
  (interactive)
  (let (c-word)
    (setq c-word (stack-pop skk-eeie-completion-stack))
    (if (string= c-word (buffer-substring-no-properties
			 skk-eeie-start (point) ))
        ;; $B%]%C%W$7$?8l$,%P%C%U%!$N%]%$%s%HD>A0$K$"$kJ8;zNs$HF1$8$@$C$?$i(B 1 $B$D(B
        ;; $B<N$F$k!#(B
        (setq c-word (stack-pop skk-eeie-completion-stack)) )
    (if c-word
	(progn
	  (delete-region (point) skk-eeie-start)
	  (insert c-word) )
      (skk-error "\"%s\"$B$GJd40$9$Y$-8+=P$78l$OB>$K$"$j$^$;$s(B"
                 "No more previous completions for \"%s\""
                 skk-eeie-completing-word ))))

(provide 'skk-eeie)
;;; skk-eeie.el ends here
